package net.wanji.datacenter.service.impl;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.datacenter.cache.CrossInfoCache;
import net.wanji.datacenter.common.PhaseEmptyEnum;
import net.wanji.datacenter.mapper.CrossStrategyResultMapper;
import net.wanji.datacenter.pojo.dto.ImbalanceAlgorithmDTO;
import net.wanji.datacenter.pojo.dto.PhaseEmptyDataDTO;
import net.wanji.datacenter.pojo.entity.CrossStrategyResultEntity;
import net.wanji.datacenter.service.CrossStrategyOptService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2024/11/23 16:14
 */
@Service
public class CrossStrategyOptServiceImpl implements CrossStrategyOptService {

    @Resource
    private CrossStrategyResultMapper crossStrategyResultMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void savePhaseEmpty(String message) throws Exception {
        try {
            ObjectMapper mapper = JacksonUtils.getInstance();
            PhaseEmptyDataDTO phaseEmptyDataDTO = mapper.readValue(message, PhaseEmptyDataDTO.class);
            if (Objects.nonNull(phaseEmptyDataDTO)) {
                PhaseEmptyDataDTO.Idx idx = phaseEmptyDataDTO.getIdx();
                String crossId = idx.getCtrId();
                Long curStamp = idx.getCurStamp();
                Integer countDown = idx.getCountDown();
                Integer rtnType = idx.getRtnType();
                // rtnType大于0时，未执行
                if (rtnType  < 0) {
                    Long signalMachineStamp = idx.getSignalMachineStamp();
                    Long createStamp = idx.getCreateStamp();
                    Date date = new Date(curStamp);
                    CrossStrategyResultEntity entity = new CrossStrategyResultEntity();
                    entity.setCrossId(crossId);
                    entity.setSignalMachineStamp(String.valueOf(signalMachineStamp));
                    // 绿灯空放
                    entity.setCurrentAlgo(1);
                    entity.setRequestTime(date);
                    entity.setIssueTime(date);
                    entity.setInsertTime(new Date(createStamp));
                    entity.setCountDown(countDown);
                    entity.setExtendTime(countDown);
                    entity.setResponseCode(rtnType);
                    entity.setRtnType(rtnType);
                    String name = CrossInfoCache.getName(crossId);
                    entity.setCrossName(name);
                    entity.setEmptyDir(PhaseEmptyEnum.getDir(rtnType));
                    entity.setEmptyTurn(PhaseEmptyEnum.getTurn(rtnType));
                    crossStrategyResultMapper.insert(entity);
                }
            }
        } catch (JsonProcessingException e) {
            throw new Exception(e);
        }

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void save(ImbalanceAlgorithmDTO dto) {
        try {
            // 未操作的不存
            // 溢出 rtnType=0 不操作 失衡未推送，目前只考虑溢出
            if (0 != dto.getRtnType()) {
                ObjectMapper mapper = JacksonUtils.getInstance();
                CrossStrategyResultEntity crossStrategyResultEntity = new CrossStrategyResultEntity();
                crossStrategyResultEntity.setCrossId(dto.getPlatformId());
                crossStrategyResultEntity.setCrossName(dto.getCrossName());
                crossStrategyResultEntity.setCurrentAlgo(dto.getCurrentAlgo());
                crossStrategyResultEntity.setRequestTime(dto.getRequestTime());
                crossStrategyResultEntity.setIssueTime(dto.getIssueTime());
                crossStrategyResultEntity.setResponseCode(dto.getResponseCode());
                crossStrategyResultEntity.setTimingPlan(mapper.writeValueAsString(dto.getTimingPlan()));
                crossStrategyResultEntity.setInsertTime(dto.getInsertTime());
                crossStrategyResultEntity.setRtnType(dto.getRtnType());
                crossStrategyResultEntity.setExtendTime(dto.getExtendTime());
                crossStrategyResultEntity.setBlockRegion(dto.getBlockRegion() + 1);
                crossStrategyResultEntity.setBlockType(dto.getBlockType());
                crossStrategyResultEntity.setCountDown(dto.getCountDown());
                crossStrategyResultEntity.setVideoStamp(dto.getVideoStamp());
                crossStrategyResultEntity.setSignalMachineStamp(dto.getSignalMachineStamp());
                crossStrategyResultEntity.setControlDir(dto.getControlDir());
                crossStrategyResultEntity.setData(mapper.writeValueAsString(dto.getData()));
                crossStrategyResultMapper.insert(crossStrategyResultEntity);
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
